extern "C" {
#include <cstdint>
namespace goodix {
typedef struct _FpInitValue {
    char template_path[260];
} FpInitValue;

typedef struct _FpRequest {
    unsigned char requestType; /* request result */
    union {
        int RequestEnrollResult;    // 单次的录入结果
        int RequestIdentifyResult;  // 认证结果
    };
    /* for enroll, return enroll progress; for identify, return match index */
    union {
        unsigned int enrollingProgress;  // 录入进度
        unsigned int matchIndex;         // 认证时，默认为 0
    };
    uint8_t matchedAccountinfo[256];
} FpRequest;

typedef void (*FpRequestCbFn)(FpRequest* request);

int FpAsyncIdentifyFingerWithoutTmpls(FpRequestCbFn callback,
                                      uint8_t*      pCurAccountInfo);
int FpAsyncIdentifyFingerWithoutAccountinfo(FpRequestCbFn callback);

enum FpEnrollResult {
    FP_ENROLL_COMPLETE            = 1,    //录入完成
    FP_ENROLL_FAIL                = 2,    //录入失败
    FP_ENROLL_PASS                = 3,    //录入通过
    FP_ENROLL_RETRY               = 100,  //重新录入
    FP_ENROLL_RETRY_TOO_SHORT     = 101,  //录入时间过短
    FP_ENROLL_RETRY_CENTER_FINGER = 102,
    FP_ENROLL_RETRY_REMOVE_FINGER = 103,
    FP_ENROLL_MORE_DATA           = 104,  //录入更多的指纹数据
    FP_ENROLL_POOR_QUALITY        = 105,  //录入的模板质量较差
    FP_ENROLL_DUPLICATE_TEMPLATE = 106,  //模板重复。录入结束返回的结果
    FP_ENROLL_MOVE_LEFT  = 107,          //录入左
    FP_ENROLL_MOVE_DOWN  = 108,          //录入下
    FP_ENROLL_MOVE_RIGHT = 109,          //录入右
    FP_ENROLL_MOVE_UP    = 110,          //录入上
};

enum FpIdentifyResult {
    FP_IDENTIFY_NO_MATCH = 1,  //异常手指匹配结果
    FP_IDENTIFY_MATCH    = 2,  //匹配成功
    FP_IDENTIFY_RETRY = 100,  //再次认证，不会计入总的失败次数中
    FP_IDENTIFY_RETRY_TOO_SHORT = 101,  //采图质量不及格，不会计入总的失败次数中
    FP_IDENTIFY_RETRY_CENTER_FINGER = 102,
    FP_IDENTIFY_RETRY_REMOVE_FINGER = 103,
    FP_IDENTIFY_MATCH_TEMPLATE_UPDATED = 104,  //更新指纹模板，补充手指特征点
};

enum FPCallType {
    FP_CALL_TYPE_FRAMEWORK = 0,  //非指纹工具调用状态
    FP_CALL_TYPE_MPTOOL    = 1   //指纹工具调用状态
};

enum FpReturnResult { FP_SUCCESS = 0, FP_ERROR = -1 };

enum FpS3Status { FP_ENTER_S3_MODE = 1, FP_EXIT_S3_MODE = 0 };

/**
   * @brief 指纹驱动在该接口中配置相关指纹信息
   *
   * @param[in] initValue 需要初始化的信息
   * @return FpReturnResult 返回成功或者失败的枚举
   */
int FpInit(FpInitValue initValue);

/**
   * @brief 启动指纹会话
   * @param[in] void
   * @return FpReturnResult 返回成功或者失败的枚举
   */
int FpStartSession(void);

int FpS3StartSession(void);

int FpStopSession(void);

int FpS3StopSession(void);

void FpExit(void);

int FpScanCancel(void);

int FpAsyncEnrollFinger(FpRequestCbFn callback);

/******************************************************************************
  * Function description:
  * 清除当前次指纹 sensor 采集的图像，在用户不想使用指纹一键开机功能时，需要在开机
  *后调用该接口将指纹图像清除，不做指纹认证。
  * Parameter description:
  * [input ] void
  *
  * Return description :
  * ERROR: return FP_ERROR
  * SUCCESS: return FP_SUCCESS ******************************************************************************/
int FpClearPovImage(void);

int FpStoreTemplate(char* path, uint8_t* accountInfo);

/******************************************************************************
  * Function description:
  * 更新指纹模板，如果指纹模板是通过加载到内存方式使用的话，在 OS 删除指纹*模板
  后，需要调用该接口，更新内存中的指纹模板。
  * Parameter description:
  * [input ] void
  *
  * Return description :
  * ERROR: return FP_ERROR
  * SUCCESS: return FP_SUCCESS ******************************************************************************/
int FpRefreshTemplates(void);

int FpDelTemplates(uint8_t** UUID, uint32_t* Count);

int FpGetVersion(unsigned char* version);

int FpEnterStopS3(int enter);

int FpUsbSuspend(void);

int FpUsbResume(void);
}  // namespace goodix
}
